from nextpy.ai import engine

from ..utils import get_llm


def test_geneach_chat_gpt():
    """Test a geneach loop with ChatGPT."""
    engine.llm = get_llm("openai:gpt-3.5-turbo")

    chat_loop = engine(
        """
{{#system~}}
You are a helpful assistant
{{~/system}}

{{~#geneach 'conversation' stop=False}}
{{#user~}}
This is great!
{{~/user}}

{{#assistant~}}
{{gen 'this.response' temperature=0 max_tokens=3}}
{{~/assistant}}
{{#if @index > 0}}{{break}}{{/if}}
{{~/geneach}}"""
    )

    out = chat_loop()
    assert len(out["conversation"]) == 2


def test_syntax_match():
    """Test a geneach loop with ChatGPT."""
    engine.llm = get_llm("openai:gpt-3.5-turbo")

    chat_loop = engine(
        """
{{~#system~}}
You are a helpful assistant
{{~/system~}}

{{~#user~}}
This is great!
{{~/user~}}

{{~#assistant~}}
Indeed
{{~/assistant~}}"""
    )

    out = chat_loop()
    assert (
        str(out)
        == "<|im_start|>system\nYou are a helpful assistant<|im_end|><|im_start|>user\nThis is great!<|im_end|><|im_start|>assistant\nIndeed<|im_end|>"
    )


def test_rest_nostream():
    engine.llm = get_llm(
        "openai:text-davinci-003",
        endpoint="https://api.openai.com/v1/completions",
        rest_call=True,
    )
    a = engine(
        """Hello,  my name is{{gen 'name' stream=False max_tokens=5}}""", stream=False
    )
    a = a()
    assert len(a["name"]) > 0


def test_rest_stream():
    engine.llm = get_llm(
        "openai:text-davinci-003",
        endpoint="https://api.openai.com/v1/completions",
        rest_call=True,
    )
    a = engine(
        """Hello,  my name is{{gen 'name' stream=True max_tokens=5}}""", stream=False
    )
    a = a()
    assert len(a["name"]) > 0


def test_rest_chat_nostream():
    engine.llm = get_llm(
        "openai:gpt-3.5-turbo",
        endpoint="https://api.openai.com/v1/chat/completions",
        rest_call=True,
    )
    prompt = engine(
        """{{#system~}}
You are a helpful assistant.
{{~/system}}
{{#user~}}
{{conversation_question}}
{{~/user}}
{{#assistant~}}
{{gen "answer" max_tokens=5 stream=False}}
{{~/assistant}}"""
    )
    prompt = prompt(conversation_question="Whats is the meaning of life??")
    assert len(prompt["answer"]) > 0


def test_rest_chat_stream():
    engine.llm = get_llm(
        "openai:gpt-3.5-turbo",
        endpoint="https://api.openai.com/v1/chat/completions",
        rest_call=True,
    )
    prompt = engine(
        """{{#system~}}
You are a helpful assistant.
{{~/system}}
{{#user~}}
{{conversation_question}}
{{~/user}}
{{#assistant~}}
{{gen "answer" max_tokens=5 stream=True}}
{{~/assistant}}"""
    )
    prompt = prompt(conversation_question="Whats is the meaning of life??")
    assert len(prompt["answer"]) > 0
